@@ -8,10 +8,10 @@ from typing import TYPE_CHECKING, Any, ClassVar, Iterable
 
 import numpy as np
 
-from ..conventions import cf_encoder
-from ..core import indexing
-from ..core.pycompat import is_duck_dask_array
-from ..core.utils import FrozenDict, NdimSizeLenMixin, is_remote_uri
+from xarray.conventions import cf_encoder
+from xarray.core import indexing
+from xarray.core.pycompat import is_duck_dask_array
+from xarray.core.utils import FrozenDict, NdimSizeLenMixin, is_remote_uri
 
 if TYPE_CHECKING:
     from io import BufferedIOBase
