@@ -1,9 +1,11 @@
+from __future__ import annotations
+
 import unicodedata
 
 import numpy as np
 
-from .. import coding
-from ..core.variable import Variable
+from xarray import coding
+from xarray.core.variable import Variable
 
 # Special characters that are permitted in netCDF names except in the
 # 0th position of the string
