@@ -1,15 +1,30 @@
+from __future__ import annotations
+
 import warnings
 from collections import defaultdict
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Hashable,
+    Iterable,
+    Mapping,
+    MutableMapping,
+    Tuple,
+    Union,
+)
 
 import numpy as np
 import pandas as pd
 
-from .coding import strings, times, variables
-from .coding.variables import SerializationWarning, pop_to
-from .core import duck_array_ops, indexing
-from .core.common import _contains_datetime_like_objects, contains_cftime_datetimes
-from .core.pycompat import is_duck_dask_array
-from .core.variable import IndexVariable, Variable, as_variable
+from xarray.coding import strings, times, variables
+from xarray.coding.variables import SerializationWarning, pop_to
+from xarray.core import duck_array_ops, indexing
+from xarray.core.common import (
+    _contains_datetime_like_objects,
+    contains_cftime_datetimes,
+)
+from xarray.core.pycompat import is_duck_dask_array
+from xarray.core.variable import IndexVariable, Variable
 
 CF_RELATED_DATA = (
     "bounds",
@@ -29,6 +44,18 @@ CF_RELATED_DATA_NEEDS_PARSING = (
 )
 
 
+if TYPE_CHECKING:
+    from xarray.backends.common import AbstractDataStore
+    from xarray.core.dataset import Dataset
+
+    T_VarTuple = Tuple[Tuple[Hashable, ...], Any, dict, dict]
+    T_Name = Union[Hashable, None]
+    T_Variables = Mapping[Any, Variable]
+    T_Attrs = MutableMapping[Any, Any]
+    T_DropVariables = Union[str, Iterable[Hashable], None]
+    T_DatasetOrAbstractstore = Union[Dataset, AbstractDataStore]
+
+
 class NativeEndiannessArray(indexing.ExplicitlyIndexedNDArrayMixin):
     """Decode arrays on the fly from non-native to native endianness
 
@@ -94,11 +121,11 @@ class BoolTypeArray(indexing.ExplicitlyIndexedNDArrayMixin):
         return np.asarray(self.array[key], dtype=self.dtype)
 
 
-def _var_as_tuple(var):
+def _var_as_tuple(var: Variable) -> T_VarTuple:
     return var.dims, var.data, var.attrs.copy(), var.encoding.copy()
 
 
-def maybe_encode_nonstring_dtype(var, name=None):
+def maybe_encode_nonstring_dtype(var: Variable, name: T_Name = None) -> Variable:
     if "dtype" in var.encoding and var.encoding["dtype"] not in ("S1", str):
         dims, data, attrs, encoding = _var_as_tuple(var)
         dtype = np.dtype(encoding.pop("dtype"))
@@ -116,13 +143,13 @@ def maybe_encode_nonstring_dtype(var, name=None):
                         SerializationWarning,
                         stacklevel=10,
                     )
-                data = duck_array_ops.around(data)[...]
+                data = np.around(data)
             data = data.astype(dtype=dtype)
-        var = Variable(dims, data, attrs, encoding)
+        var = Variable(dims, data, attrs, encoding, fastpath=True)
     return var
 
 
-def maybe_default_fill_value(var):
+def maybe_default_fill_value(var: Variable) -> Variable:
     # make NaN the fill value for float types:
     if (
         "_FillValue" not in var.attrs
@@ -133,7 +160,7 @@ def maybe_default_fill_value(var):
     return var
 
 
-def maybe_encode_bools(var):
+def maybe_encode_bools(var: Variable) -> Variable:
     if (
         (var.dtype == bool)
         and ("dtype" not in var.encoding)
@@ -141,12 +168,12 @@ def maybe_encode_bools(var):
     ):
         dims, data, attrs, encoding = _var_as_tuple(var)
         attrs["dtype"] = "bool"
-        data = data.astype(dtype="i1", copy=True)
-        var = Variable(dims, data, attrs, encoding)
+        data = duck_array_ops.astype(data, dtype="i1", copy=True)
+        var = Variable(dims, data, attrs, encoding, fastpath=True)
     return var
 
 
-def _infer_dtype(array, name=None):
+def _infer_dtype(array, name: T_Name = None) -> np.dtype:
     """Given an object array with no missing values, infer its dtype from its
     first element
     """
@@ -174,7 +201,7 @@ def _infer_dtype(array, name=None):
     )
 
 
-def ensure_not_multiindex(var, name=None):
+def ensure_not_multiindex(var: Variable, name: T_Name = None) -> None:
     if isinstance(var, IndexVariable) and isinstance(var.to_index(), pd.MultiIndex):
         raise NotImplementedError(
             "variable {!r} is a MultiIndex, which cannot yet be "
@@ -185,7 +212,7 @@ def ensure_not_multiindex(var, name=None):
         )
 
 
-def _copy_with_dtype(data, dtype):
+def _copy_with_dtype(data, dtype: np.typing.DTypeLike):
     """Create a copy of an array with the given dtype.
 
     We use this instead of np.array() to ensure that custom object dtypes end
@@ -196,7 +223,7 @@ def _copy_with_dtype(data, dtype):
     return result
 
 
-def ensure_dtype_not_object(var, name=None):
+def ensure_dtype_not_object(var: Variable, name: T_Name = None) -> Variable:
     # TODO: move this from conventions to backends? (it's not CF related)
     if var.dtype.kind == "O":
         dims, data, attrs, encoding = _var_as_tuple(var)
@@ -221,6 +248,7 @@ def ensure_dtype_not_object(var, name=None):
             # There is no safe bit-pattern for NA in typical binary string
             # formats, we so can't set a fill_value. Unfortunately, this means
             # we can't distinguish between missing values and empty strings.
+            fill_value: bytes | str
             if strings.is_bytes_dtype(inferred_dtype):
                 fill_value = b""
             elif strings.is_unicode_dtype(inferred_dtype):
@@ -237,11 +265,13 @@ def ensure_dtype_not_object(var, name=None):
             data = _copy_with_dtype(data, dtype=_infer_dtype(data, name))
 
         assert data.dtype.kind != "O" or data.dtype.metadata
-        var = Variable(dims, data, attrs, encoding)
+        var = Variable(dims, data, attrs, encoding, fastpath=True)
     return var
 
 
-def encode_cf_variable(var, needs_copy=True, name=None):
+def encode_cf_variable(
+    var: Variable, needs_copy: bool = True, name: T_Name = None
+) -> Variable:
     """
     Converts an Variable into an Variable which follows some
     of the CF conventions:
@@ -284,16 +314,16 @@ def encode_cf_variable(var, needs_copy=True, name=None):
 
 
 def decode_cf_variable(
-    name,
-    var,
-    concat_characters=True,
-    mask_and_scale=True,
-    decode_times=True,
-    decode_endianness=True,
-    stack_char_dim=True,
-    use_cftime=None,
-    decode_timedelta=None,
-):
+    name: Hashable,
+    var: Variable,
+    concat_characters: bool = True,
+    mask_and_scale: bool = True,
+    decode_times: bool = True,
+    decode_endianness: bool = True,
+    stack_char_dim: bool = True,
+    use_cftime: bool | None = None,
+    decode_timedelta: bool | None = None,
+) -> Variable:
     """
     Decodes a variable which may hold CF encoded information.
 
@@ -339,8 +369,6 @@ def decode_cf_variable(
     out : Variable
         A variable holding the decoded equivalent of var.
     """
-    var = as_variable(var)
-
     # Ensure datetime-like Variables are passed through unmodified (GH 6453)
     if _contains_datetime_like_objects(var):
         return var
@@ -385,10 +413,10 @@ def decode_cf_variable(
     if not is_duck_dask_array(data):
         data = indexing.LazilyIndexedArray(data)
 
-    return Variable(dimensions, data, attributes, encoding=encoding)
+    return Variable(dimensions, data, attributes, encoding=encoding, fastpath=True)
 
 
-def _update_bounds_attributes(variables):
+def _update_bounds_attributes(variables: T_Variables) -> None:
     """Adds time attributes to time bounds variables.
 
     Variables handling time bounds ("Cell boundaries" in the CF
@@ -417,7 +445,7 @@ def _update_bounds_attributes(variables):
                     bounds_attrs.setdefault("calendar", attrs["calendar"])
 
 
-def _update_bounds_encoding(variables):
+def _update_bounds_encoding(variables: T_Variables) -> None:
     """Adds time encoding to time bounds variables.
 
     Variables handling time bounds ("Cell boundaries" in the CF
@@ -434,7 +462,7 @@ def _update_bounds_encoding(variables):
     """
 
     # For all time variables with bounds
-    for v in variables.values():
+    for name, v in variables.items():
         attrs = v.attrs
         encoding = v.encoding
         has_date_units = "units" in encoding and "since" in encoding["units"]
@@ -455,7 +483,7 @@ def _update_bounds_encoding(variables):
                 "and '{1}' will be determined independently "
                 "and may not be equal, counter to CF-conventions. "
                 "If this is a concern, specify a units encoding for "
-                "'{0}' before writing to a file.".format(v.name, attrs["bounds"]),
+                "'{0}' before writing to a file.".format(name, attrs["bounds"]),
                 UserWarning,
             )
 
@@ -468,16 +496,16 @@ def _update_bounds_encoding(variables):
 
 
 def decode_cf_variables(
-    variables,
-    attributes,
-    concat_characters=True,
-    mask_and_scale=True,
-    decode_times=True,
-    decode_coords=True,
-    drop_variables=None,
-    use_cftime=None,
-    decode_timedelta=None,
-):
+    variables: T_Variables,
+    attributes: T_Attrs,
+    concat_characters: bool = True,
+    mask_and_scale: bool = True,
+    decode_times: bool = True,
+    decode_coords: bool = True,
+    drop_variables: T_DropVariables = None,
+    use_cftime: bool | None = None,
+    decode_timedelta: bool | None = None,
+) -> tuple[T_Variables, T_Attrs, set[Hashable]]:
     """
     Decode several CF encoded variables.
 
@@ -488,7 +516,7 @@ def decode_cf_variables(
         for d in v.dims:
             dimensions_used_by[d].append(v)
 
-    def stackable(dim):
+    def stackable(dim: Hashable) -> bool:
         # figure out if a dimension can be concatenated over
         if dim in variables:
             return False
@@ -574,23 +602,24 @@ def decode_cf_variables(
                         )
                     del var_attrs[attr_name]
 
-    if decode_coords and "coordinates" in attributes:
+    if decode_coords and isinstance(attributes.get("coordinates", None), str):
         attributes = dict(attributes)
-        coord_names.update(attributes.pop("coordinates").split())
+        crds = attributes.pop("coordinates")
+        coord_names.update(crds.split())
 
     return new_vars, attributes, coord_names
 
 
 def decode_cf(
-    obj,
-    concat_characters=True,
-    mask_and_scale=True,
-    decode_times=True,
-    decode_coords=True,
-    drop_variables=None,
-    use_cftime=None,
-    decode_timedelta=None,
-):
+    obj: T_DatasetOrAbstractstore,
+    concat_characters: bool = True,
+    mask_and_scale: bool = True,
+    decode_times: bool = True,
+    decode_coords: bool = True,
+    drop_variables: T_DropVariables = None,
+    use_cftime: bool | None = None,
+    decode_timedelta: bool | None = None,
+) -> Dataset:
     """Decode the given Dataset or Datastore according to CF conventions into
     a new Dataset.
 
@@ -639,9 +668,11 @@ def decode_cf(
     -------
     decoded : Dataset
     """
-    from .backends.common import AbstractDataStore
-    from .core.dataset import Dataset
+    from xarray.backends.common import AbstractDataStore
+    from xarray.core.dataset import Dataset
 
+    vars: T_Variables
+    attrs: T_Attrs
     if isinstance(obj, Dataset):
         vars = obj._variables
         attrs = obj.attrs
@@ -676,12 +707,12 @@ def decode_cf(
 
 
 def cf_decoder(
-    variables,
-    attributes,
-    concat_characters=True,
-    mask_and_scale=True,
-    decode_times=True,
-):
+    variables: T_Variables,
+    attributes: T_Attrs,
+    concat_characters: bool = True,
+    mask_and_scale: bool = True,
+    decode_times: bool = True,
+) -> tuple[T_Variables, T_Attrs]:
     """
     Decode a set of CF encoded variables and attributes.
 
