@@ -1,8 +1,10 @@
+from __future__ import annotations
+
 import functools
 
 import numpy as np
 
-from . import utils
+from xarray.core import utils
 
 # Use as a sentinel value to indicate a dtype appropriate NA value.
 NA = utils.ReprObject("<NA>")
